#include "fastmath_num.h"
#include <num/num.h>

libnum::num16 fp_sqrt16(libnum::num16 x) {
  uint16_t b = 1 << 14, q = 0, r = x.v;
  while (b > r)
    b >>= 2;
  while (b > 0) {
    uint16_t t = q + b;
    q >>= 1;
    if (r >= t) {
      r -= t;
      q += b;
    }
    b >>= 2;
  }
  libnum::num16 ret;
  ret.v = q << 4;
  return ret;
}

libnum::num16 fp_cos16(libnum::num16 a) {
  if (a < 0)
    a = -a;
  a = a % libnum::num16(6.28319);
  if (a > libnum::num16(3.14159))
    a -= libnum::num16(6.28319);

  libnum::num16 u = 1.0;
  int p = 7;
  for (p = 2 * p - 1; p >= 1; p -= 2)
    u = libnum::num16(1) - a * a / (p * p + p) * u;
  return u;
}

libnum::num16 fp_sin16(libnum::num16 a) {
  return fp_cos16(a - libnum::num16(1.57080));
}

libnum::num16 fp_tan16(libnum::num16 a) {
  libnum::num16 sine = fp_sin16(a);
  libnum::num16 cosine = fp_cos16(a);
  libnum::num16 ret;

  if (sine == 0 && cosine == libnum::num16(-1))
    ret.v = 0x8000;
  else if (sine == 0 && cosine == libnum::num16(1))
    ret.v = 0x7FFF;
  else
    ret = sine / cosine;

  return ret;
}

libnum::num32 fp_sqrt32(libnum::num32 x) {
  uint32_t b = 1 << 30, q = 0, r = x.v;
  while (b > r)
    b >>= 2;
  while (b > 0) {
    uint32_t t = q + b;
    q >>= 1;
    if (r >= t) {
      r -= t;
      q += b;
    }
    b >>= 2;
  }
  libnum::num32 ret;
  ret.v = q << 8;
  return ret;
}

libnum::num32 fp_cos32(libnum::num32 a) {
  if (a < 0)
    a = -a;
  a = a % libnum::num32(6.28319);
  if (a > libnum::num32(3.14159))
    a -= libnum::num32(6.28319);

  libnum::num32 u = 1.0;
  int p = 7;
  for (p = 2 * p - 1; p >= 1; p -= 2)
    u = libnum::num32(1) - a * a / (p * p + p) * u;
  return u;
}

libnum::num32 fp_sin32(libnum::num32 a) {
  return fp_cos32(a - libnum::num32(1.57080));
}

libnum::num32 fp_tan32(libnum::num32 a) {
  libnum::num32 sine = fp_sin32(a);
  libnum::num32 cosine = fp_cos32(a);
  libnum::num32 ret;

  if (sine == 0 && cosine == libnum::num32(-1))
    ret.v = 0x80000000;
  else if (sine == 0 && cosine == libnum::num32(1))
    ret.v = 0x7FFFFFFF;
  else
    ret = sine / cosine;

  return ret;
}