#include "matrix3D_num16.h"
#include "fastmath_num.h"
#include "vector3D_num16.h"
#include <num/num.h>
#include <stdint.h>

v3d_16 Multiply(m3d_16 m, v3d_16 i) {
  v3d_16 ret;

  ret.x = i.x * m.m[0][0] + i.y * m.m[1][0] + i.z * m.m[2][0] + i.w * m.m[3][0];
  ret.y = i.x * m.m[0][1] + i.y * m.m[1][1] + i.z * m.m[2][1] + i.w * m.m[3][1];
  ret.z = i.x * m.m[0][2] + i.y * m.m[1][2] + i.z * m.m[2][2] + i.w * m.m[3][2];
  ret.w = i.x * m.m[0][3] + i.y * m.m[1][3] + i.z * m.m[2][3] + i.w * m.m[3][3];

  return ret;
}

v3d_16 Multiply(m3d_16 &m, v3d_16 &i) {
  v3d_16 ret;

  ret.x = i.x * m.m[0][0] + i.y * m.m[1][0] + i.z * m.m[2][0] + i.w * m.m[3][0];
  ret.y = i.x * m.m[0][1] + i.y * m.m[1][1] + i.z * m.m[2][1] + i.w * m.m[3][1];
  ret.z = i.x * m.m[0][2] + i.y * m.m[1][2] + i.z * m.m[2][2] + i.w * m.m[3][2];
  ret.w = i.x * m.m[0][3] + i.y * m.m[1][3] + i.z * m.m[2][3] + i.w * m.m[3][3];

  return ret;
}

m3d_16 Multiply(m3d_16 m1, m3d_16 m2) {
  m3d_16 ret;
  for (int c = 0; c < 4; c++)
    for (int r = 0; r < 4; r++)
      ret.m[r][c] = m1.m[r][0] * m2.m[0][c] + m1.m[r][1] * m2.m[1][c] +
                    m1.m[r][2] * m2.m[2][c] + m1.m[r][3] * m2.m[3][c];
  return ret;
}

m3d_16 Multiply(m3d_16 &m1, m3d_16 &m2) {
  m3d_16 ret;
  for (int c = 0; c < 4; c++)
    for (int r = 0; r < 4; r++)
      ret.m[r][c] = m1.m[r][0] * m2.m[0][c] + m1.m[r][1] * m2.m[1][c] +
                    m1.m[r][2] * m2.m[2][c] + m1.m[r][3] * m2.m[3][c];
  return ret;
}

m3d_16 QuickInverse(m3d_16 m) {
  m3d_16 ret;
  ret.m[0][0] = m.m[0][0];
  ret.m[0][1] = m.m[1][0];
  ret.m[0][2] = m.m[2][0];
  ret.m[0][3] = libnum::num16(0);
  ret.m[1][0] = m.m[0][1];
  ret.m[1][1] = m.m[1][1];
  ret.m[1][2] = m.m[2][1];
  ret.m[1][3] = libnum::num16(0);
  ret.m[2][0] = m.m[0][2];
  ret.m[2][1] = m.m[1][2];
  ret.m[2][2] = m.m[2][2];
  ret.m[2][3] = libnum::num16(0);
  ret.m[3][0] = -(m.m[3][0] * ret.m[0][0] + m.m[3][1] * ret.m[1][0] +
                  m.m[3][2] * ret.m[2][0]);
  ret.m[3][1] = -(m.m[3][0] * ret.m[0][1] + m.m[3][1] * ret.m[1][1] +
                  m.m[3][2] * ret.m[2][1]);
  ret.m[3][2] = -(m.m[3][0] * ret.m[0][2] + m.m[3][1] * ret.m[1][2] +
                  m.m[3][2] * ret.m[2][2]);
  ret.m[3][3] = libnum::num16(1);
  return ret;
}

m3d_16 QuickInverse(m3d_16 &m) {
  m3d_16 ret;
  ret.m[0][0] = m.m[0][0];
  ret.m[0][1] = m.m[1][0];
  ret.m[0][2] = m.m[2][0];
  ret.m[0][3] = libnum::num16(0);
  ret.m[1][0] = m.m[0][1];
  ret.m[1][1] = m.m[1][1];
  ret.m[1][2] = m.m[2][1];
  ret.m[1][3] = libnum::num16(0);
  ret.m[2][0] = m.m[0][2];
  ret.m[2][1] = m.m[1][2];
  ret.m[2][2] = m.m[2][2];
  ret.m[2][3] = libnum::num16(0);
  ret.m[3][0] = -(m.m[3][0] * ret.m[0][0] + m.m[3][1] * ret.m[1][0] +
                  m.m[3][2] * ret.m[2][0]);
  ret.m[3][1] = -(m.m[3][0] * ret.m[0][1] + m.m[3][1] * ret.m[1][1] +
                  m.m[3][2] * ret.m[2][1]);
  ret.m[3][2] = -(m.m[3][0] * ret.m[0][2] + m.m[3][1] * ret.m[1][2] +
                  m.m[3][2] * ret.m[2][2]);
  ret.m[3][3] = libnum::num16(1);
  return ret;
}