#ifndef MATRIX3D_NUM16
#define MATRIX3D_NUM16

#include "fastmath_num.h"
#include "vector3D_num16.h"
#include <num/num.h>
#include <stdint.h>

class m3d_16 {
public:
  void MakeIdentity(void) {
    m[0][0] = libnum::num16(1);
    m[1][1] = libnum::num16(1);
    m[2][2] = libnum::num16(1);
    m[3][3] = libnum::num16(1);
  }

  void MakeRotationX(libnum::num16 angle) {
    libnum::num16 cosine = fp_cos16(angle);
    libnum::num16 sine = fp_sin16(angle);
    m[0][0] = libnum::num16(1);
    m[1][1] = cosine;
    m[1][2] = sine;
    m[2][1] = -sine;
    m[2][2] = cosine;
    m[3][3] = libnum::num16(1);
  }

  void MakeRotationY(libnum::num16 angle) {
    libnum::num16 cosine = fp_cos16(angle);
    libnum::num16 sine = fp_sin16(angle);
    m[0][0] = cosine;
    m[0][2] = sine;
    m[2][0] = -sine;
    m[1][1] = libnum::num16(1);
    m[2][2] = cosine;
    m[3][3] = libnum::num16(1);
  }

  void MakeRotationZ(libnum::num16 angle) {
    libnum::num16 cosine = fp_cos16(angle);
    libnum::num16 sine = fp_sin16(angle);
    m[0][0] = cosine;
    m[0][1] = sine;
    m[1][0] = -sine;
    m[1][1] = cosine;
    m[2][2] = libnum::num16(1);
    m[3][3] = libnum::num16(1);
  }

  void MakeTranslation(v3d_16 vecT) {
    m[0][0] = libnum::num16(1);
    m[1][1] = libnum::num16(1);
    m[2][2] = libnum::num16(1);
    m[3][3] = libnum::num16(1);
    m[3][0] = vecT.x;
    m[3][1] = vecT.y;
    m[3][2] = vecT.z;
  }

  void MakeProjection(libnum::num16 fov_degree, libnum::num16 aspect_ratio,
                      libnum::num16 near_distance, libnum::num16 far_distance) {
    libnum::num16 fov_rad =
        libnum::num16(1) /
        fp_tan16(fov_degree * libnum::num16(0.5f) / libnum::num16(180) *
                 libnum::num16(3.14159f));

    m[0][0] = aspect_ratio * fov_rad;
    m[1][1] = fov_rad;
    m[2][2] = far_distance / (far_distance - near_distance);
    m[3][2] = (-far_distance * near_distance) / (far_distance - near_distance);
    m[2][3] = libnum::num16(1);
    m[3][3] = libnum::num16(0);
  }

  void MakeScale(libnum::num16 scaleX, libnum::num16 scaleY,
                 libnum::num16 scaleZ) {
    m[0][0] = scaleX;
    m[1][1] = scaleY;
    m[2][2] = scaleZ;
    m[3][3] = libnum::num16(1);
  }

  libnum::num16 m[4][4] = {0};
};

v3d_16 Multiply(m3d_16 m, v3d_16 i);
v3d_16 Multiply(m3d_16 &m, v3d_16 &i);

m3d_16 Multiply(m3d_16 m1, m3d_16 m2);
m3d_16 Multiply(m3d_16 &m1, m3d_16 &m2);

m3d_16 QuickInverse(m3d_16 m);
m3d_16 QuickInverse(m3d_16 &m);

#endif