#include "vector3D_num16.h"
#include "fastmath_num.h"

v3d_16::v3d_16() {
  this->x = libnum::num16(0);
  this->y = libnum::num16(0);
  this->z = libnum::num16(0);
  this->w = libnum::num16(1);
}

v3d_16::v3d_16(int x, int y, int z) {
  this->x = libnum::num16(x);
  this->y = libnum::num16(y);
  this->z = libnum::num16(z);
  this->w = libnum::num16(1);
}

v3d_16::v3d_16(float x, float y, float z) {
  this->x = libnum::num16(x);
  this->y = libnum::num16(y);
  this->z = libnum::num16(z);
  this->w = libnum::num16(1);
}

v3d_16::v3d_16(libnum::num16 x, libnum::num16 y, libnum::num16 z) {
  this->x = x;
  this->y = y;
  this->z = z;
  this->w = libnum::num16(1);
}

v3d_16::v3d_16(const v3d_16 &v) {
  this->x = v.x;
  this->y = v.y;
  this->z = v.z;
  this->w = libnum::num16(1);
}

v3d_16::~v3d_16() {}

void v3d_16::Set(v3d_16 v) {
  this->x = v.x;
  this->y = v.y;
  this->z = v.z;
  this->w = libnum::num16(1);
}

void v3d_16::Normalise(void) {
  libnum::num16 len = this->Length();
  this->x /= len;
  this->y /= len;
  this->z /= len;
}

v3d_16 v3d_16::Clone(void) {
  v3d_16 NewVector(this->x, this->y, this->z);
  return NewVector;
}

v3d_16 v3d_16::MakeVector(v3d_16 A, v3d_16 B) {
  v3d_16 NewVector(B.x - A.x, B.y - A.y, B.z - A.z);
  return NewVector;
}

void v3d_16::AddVectors(v3d_16 a, v3d_16 b) {
  this->x = a.x + b.x;
  this->y = a.y + b.y;
  this->z = a.z + b.z;
}

void v3d_16::Add(v3d_16 v, libnum::num16 scale) {
  this->x += v.x * scale;
  this->y += v.y * scale;
  this->z += v.z * scale;
}

void v3d_16::SubtractVectors(v3d_16 a, v3d_16 b) {
  this->x = a.x - b.x;
  this->y = a.y - b.y;
  this->z = a.z - b.z;
}

void v3d_16::Subtract(v3d_16 v, libnum::num16 scale) {
  this->x -= v.x * scale;
  this->y -= v.y * scale;
  this->z -= v.y * scale;
}

libnum::num16 v3d_16::Length(void) {
  return fp_sqrt16(this->x * this->x + this->y * this->y + this->z * this->z);
}

void v3d_16::Scale(libnum::num16 scale) {
  this->x *= scale;
  this->y *= scale;
  this->z *= scale;
}

libnum::num16 v3d_16::Dot(v3d_16 v) {
  return (this->x * v.x + this->y * v.y + this->z * v.z);
}

v3d_16 CrossProduct(v3d_16 v1, v3d_16 v2) {
  v3d_16 vret;
  vret.x = v1.y * v2.z - v1.z * v2.y;
  vret.y = v1.z * v2.x - v1.x * v2.z;
  vret.z = v1.x * v2.y - v1.y * v2.x;
  return vret;
}

v3d_16 CrossProduct(v3d_16 &v1, v3d_16 &v2) {
  v3d_16 vret;
  vret.x = v1.y * v2.z - v1.z * v2.y;
  vret.y = v1.z * v2.x - v1.x * v2.z;
  vret.z = v1.x * v2.y - v1.y * v2.x;
  return vret;
}