#ifndef VECTOR3D_NUM16
#define VECTOR3D_NUM16

#include <num/num.h>
#include <stdint.h>

class v3d_16 {
public:
  v3d_16();
  v3d_16(float x, float y, float z);
  v3d_16(int x, int y, int z);
  v3d_16(libnum::num16 x, libnum::num16 y, libnum::num16 z);
  v3d_16(const v3d_16 &v);
  ~v3d_16();

  void Set(v3d_16 v);
  void Normalise(void);

  v3d_16 Clone(void);
  v3d_16 MakeVector(v3d_16 A, v3d_16 B);

  void AddVectors(v3d_16 a, v3d_16 b);
  void Add(v3d_16 v, libnum::num16 scale);
  void SubtractVectors(v3d_16 a, v3d_16 b);
  void Subtract(v3d_16 v, libnum::num16 scale);

  libnum::num16 Length(void);
  void Scale(libnum::num16 scale);

  libnum::num16 Dot(v3d_16 v);

  /* overloading of most interesting operators */
  libnum::num16 operator[](uint8_t pos) {
    return pos == 0 ? x : pos == 1 ? y : pos == 2 ? z : w;
  }

  v3d_16 &operator=(const v3d_16 &v) {
    this->x = v.x;
    this->y = v.y;
    this->z = v.z;
    this->w = v.w;
    return *this;
  }

  v3d_16 operator+(const v3d_16 &v) const {
    return v3d_16(x + v.x, y + v.y, z + v.z);
  }

  v3d_16 operator-(const v3d_16 &v) const {
    return v3d_16(x - v.x, y - v.y, z - v.z);
  }

  v3d_16 &operator+=(v3d_16 const &other) {
    this->x += other.x;
    this->y += other.y;
    this->z += other.z;
    return *this;
  }

  v3d_16 operator-() const { return (v3d_16(-x, -y, -z)); }

  v3d_16 operator+() const { return *this; }

  v3d_16 &operator-=(v3d_16 const &other) {
    this->x -= other.x;
    this->y -= other.y;
    this->z -= other.z;
    return *this;
  }

  v3d_16 &operator*=(libnum::num16 scale) {
    this->x *= scale;
    this->y *= scale;
    this->z *= scale;
    return *this;
  }

  v3d_16 &operator/=(libnum::num16 scale) {
    this->x /= scale;
    this->y /= scale;
    this->z /= scale;
    return *this;
  }

  friend v3d_16 operator*(libnum::num16 scale, v3d_16 const &v) {
    v3d_16 r;
    r.x = v.x * scale;
    r.y = v.y * scale;
    r.z = v.z * scale;
    return r;
  }

  friend v3d_16 operator*(v3d_16 const &v, libnum::num16 scale) {
    v3d_16 r;
    r.x = v.x * scale;
    r.y = v.y * scale;
    r.z = v.z * scale;
    return r;
  }

  friend v3d_16 operator/(v3d_16 const &v, libnum::num16 scale) {
    v3d_16 r;
    r.x = v.x / scale;
    r.y = v.y / scale;
    r.z = v.z / scale;
    return r;
  }

  /* usual (x,y,z) coordinates */
  libnum::num16 x = libnum::num16(0);
  libnum::num16 y = libnum::num16(0);
  libnum::num16 z = libnum::num16(0);
  /* w is used for mat4_4 calculations that can be used to treat translations */
  libnum::num16 w = libnum::num16(1);
};

v3d_16 CrossProduct(v3d_16 v1, v3d_16 v2);
v3d_16 CrossProduct(v3d_16 &v1, v3d_16 &v2);

#endif